#include <EEPROM.h>
#include <avr/sleep.h>
#include <avr/power.h>
#include <avr/wdt.h>

const int PIN_IRLED = 3;
const int PIN_NTC_VERSORGUNG = 9;
const int PIN_NTC_MESSUNG = A5;

const int messung_alle = 8; //Messe alle 16 Sekunden

void setup() {

  pinMode(PIN_IRLED, INPUT);
  digitalWrite(PIN_IRLED, HIGH);  
  digitalWrite(PIN_NTC_VERSORGUNG, LOW);
  pinMode(PIN_NTC_VERSORGUNG, INPUT);
  pinMode(PIN_NTC_MESSUNG, INPUT);

  //Stelle Timer2 auf eine Frequenz von ~36KHz
  TCCR2A = B01100011;
  TCCR2B = B00001001;
  OCR2A = 27;
  OCR2B = 8;

  //Bereite den Watchdog-Timer vor
  MCUSR &= ~(1 << WDRF);
  WDTCSR |= (1 << WDCE) | (1 << WDE);
  WDTCSR = B01100001;
}

void loop() {
  int temperatur = messe_Temperatur();
  uebertrage_Messwert(temperatur);
  //schreibe_in_EEPROM(temperatur);

  for (int i = 0; i < messung_alle / 8; i++) {
    schlafe();
  }
}

const int sensor_ID = 1;

void uebertrage_Messwert(int messwert) {
  long nachricht = 0;
  nachricht |= messwert & 0xffff;
  nachricht |= (long)sensor_ID << 16;

  for (int i = 0; i < 3; i++) {
    sende_IR_Code(nachricht, 24);
    delay(114);
  }
}

int messe_Temperatur() {
  //Versorge den Thermistor mit Spannung
  pinMode(PIN_NTC_VERSORGUNG, OUTPUT);
  delay(50);
  double temperatur = berechne_Temperatur(berechne_Widerstand());
  //Versorge den Thermistor nicht mehr mit Spannung
  pinMode(PIN_NTC_VERSORGUNG, INPUT);
  return (int)(temperatur * 10);
}

void schlafe() {
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  
  ADCSRA &= B01111111;//schalte den ADC aus
  ADMUX &= B10111111;//Schalte die Interne Spannungsreferenz aus
  
  MCUCR |= B01100000; //Setze das Sicherungsbit
  MCUCR |= B01000000; //Setze das BOD-Disable-Bit
  MCUCR &= B11011111; //Lösche das Sicherungsbit
  sleep_enable();
  sleep_mode();
  sleep_disable();
  
  ADCSRA |= B10000000;//schalte den ADC ein
  ADMUX |= B01000000;//Schalte die interne Spannungsreferenz ein

}

int adresse = 0;
void schreibe_in_EEPROM(int wert) {
  int lower = wert & 0xFF;
  int higher = wert >> 8;
  EEPROM.write(adresse, lower);
  EEPROM.write(adresse + 1, higher);
  EEPROM.write(adresse + 2, 0);
  EEPROM.write(adresse + 3, 0);
  adresse += 2;
  if (adresse > 1024 – 4)
    adresse = 0;
}

const int tHalfBit = 90;
void sende_IR_Code(unsigned long value, byte bitLength) {

  //Übertrage B11, um die Übertragung zu initialisieren
  for (int i = 0; i < 2; i++) {
    pinMode(PIN_IRLED, INPUT); //IR ausschalten
    delayMicroseconds(tHalfBit);
    pinMode(PIN_IRLED, OUTPUT); //IR einschalten
    delayMicroseconds(tHalfBit);
  }
  
  //Übertrage jedes Bit der Nachricht
  for (int i = bitLength - 1; i >= 0; i--) {
    if ((value >> i) % 2 == 1) { //Soll 1 übertragen werden?
      pinMode(PIN_IRLED, INPUT);   //IR ausschalten
      delayMicroseconds(tHalfBit);
      pinMode(PIN_IRLED, OUTPUT);  //IR einschalten
      delayMicroseconds(tHalfBit);
    } else {                     //Soll 0 übertragen werden?
      pinMode(PIN_IRLED, OUTPUT);  //IR einschalten
      delayMicroseconds(tHalfBit);
      pinMode(PIN_IRLED, INPUT);   //IR ausschalten
      delayMicroseconds(tHalfBit);
    }
  }
  pinMode(PIN_IRLED, INPUT); //IR ausschalten
}

//Aus dem Kapitel "analoger Temperatursensor"
const double R2 = 2000;
double berechne_Widerstand() {
  //Spannung, die über R1 abfällt
  double U1 = analogRead(A5) / 1024.0 * 5.0;

  //Berechnung von R1 über Umformung
  //  der Spannungsteiler-Formel
  double R1 = (R2 * U1) / (5.0 - U1);
  return R1;
}

//Aus dem Kapitel "analoger Temperatursensor"
const double R_ref = 2000;
double berechne_Temperatur(double R1) {
  //Parameter des Thermistors aus dem Datenblatt
  double A, B, C, D;

  //Um welchen Faktor unterscheidet sich der gemessene Wert
  //vom Referenzwer von 2kOhm?
  double R = R1 / R_ref;

  //Fallunterscheidung: Die Berechnung variiert,
  // je nach betrachtetem Temperaturbereich
  if (R > 1.0) {
    //Fall 1: Temperatur < 25°C
    A = 3.354016e-3;
    B = 2.90967e-4;
    C = 1.632136e-6;
    D = 7.1922e-8;
  } else {
    //Fall 2: Temperatur >= 25°C
    A = 3.354016e-3;
    B = 2.933908e-4;
    C = 3.494314e-6;
    D = -7.71269e-7;
  }

  //Einsetzen der Werte in die Formel und Berechnung
  //der Temperatur
  double T = 1.0 / (A
                    + B * log(R)
                    + C * (log(R) * log(R))
                    + D * (log(R) * log(R) * log(R)));
  return T - 273.15;
}
