# -*- coding: utf-8 -*- 
#!/usr/bin/python
#------------------------------------------------------------------------   
# Robotik mit dem Raspberry Pi
# E.F.Engelhardt, Franzis Verlag, 2014 
# ----------------------------------------------------------------------- 
# Das Skript nutzt die Adafruit PWM-Erweiterung 16 Kanal 
# und steuert damit Servomotor 0-15 an
#
# lego-adafruit-pwm-01.py
# -----------------------------------------------------------------------

from Adafruit_PWM_Servo_Driver import PWM
import time
import serial
import array
import os
import sys
import threading
import struct
#--------------------------------------------------------------------------------
pwm = PWM(0x40, debug=True)
#--------------------------------------------------------------------------------
servoMin = 150  # Min pulse length out of 4096
servoMax = 600  # Max pulse length out of 4096 
servoMid = ((servoMax - servoMin) / 2 + servoMin)
offset = (servoMid * 2 / 3)/10
servoMid = int(round(servoMid+offset,0))
step = servoMid
#--------------------------------------------------------------------------------
class _Getcharacter:
    def __init__(self):
        self.impl = _Getchar()
    def __call__(self):
        bytes = self.impl()
        if bytes == b'\x03':
            raise KeyboardInterrupt
        elif bytes == b'\x04':
            raise EOFError
        return bytes
# --------------------------------------------------------------------------------
class _Getchar:
    def __init__(self):
        import tty
    def __call__(self):
        import tty
        import termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch
# --------------------------------------------------------------------------------
getch = _Getcharacter()
# --------------------------------------------------------------------------------
def showControl():
    os.system('clear')
    print '|----------------------------------------------------------------|'
    print '| [ LEGO-SERVO ] A = Schneller, D = Langsamer                    |'
    print '|                S = Alle Motoren STOP, X = Programmende         |'
    print '|                                                                |'
    print '| [ MOTOR-NR:', servoMotor+1,'] Y = Motor > , W = Motor < (Default 1,Max 16)   |'
    print '|----------------------------------------------------------------|'
    print '| Pulselaenge: ', step, ' Max = ', servoMax, ' Min = ', servoMin,'                     |'
    print '| Bei Wert um ', servoMid, ' Richtungswechsel                             |'
    print '|----------------------------------------------------------------|'
# --------------------------------------------------------------------------------
# MAIN
#--------------------------------------------------------------------------------

if __name__ == '__main__':
    pwm.setPWMFreq(60)                        # frequenz 60 Hz
    servoMotor=0 # Motoranschluss auf AdafruitPort
    showControl(); #print 'A = Schneller, D = Langsamer, X = Ende'
    inkey = _Getcharacter()
    while True:
        k=inkey()
        print(k[0])
        # 'a' ord: 97
        if (k[0] == 'a'):
            #
            #    fast motor
            # 
            step = step + 10
	    if step < servoMin:
               step = servoMid;
            if step > servoMax:
               step = servoMax;
            pwm.setPWM(servoMotor, 0, step)
            time.sleep(0.1)
            showControl();
        # 'd' ord: 100
        elif(k[0] == 'd'):
            # 
            #    slow motor 
            #
            step = step - 10;
            if step < servoMin:
               step = servoMid;
            if step > servoMax:
               step = servoMax;
            pwm.setPWM(servoMotor, 0, step)
            time.sleep(0.1)
            showControl();
        # 'w' up ord: 119
        elif(k[0] == 'w'):
            #
            #    next motor 
            #
            servoMotor = servoMotor + 1;
            if servoMotor > 15:
                servoMotor = 0;
            pwm.setPWM(servoMotor, 0, servoMid)          
            time.sleep(0.1)
            step = servoMid
            showControl();
        # 'y' down ord: 121
        elif(k[0] == 'y'):
            #
            #    prev motor
            #
            servoMotor = servoMotor - 1;
            if servoMotor < 0:
                servoMotor = 15;
            pwm.setPWM(servoMotor, 0, servoMid)
            time.sleep(0.1)
            step = servoMid
            showControl();
        elif(k[0] == 's'):
            #
            #    alles stop
            #
            stopStep = 0
            for servoMotor in range(0,16):
                pwm.setPWM(servoMotor, 0, stopStep)
                time.sleep(0.1)
            step = 0
            showControl();
        # 'x' exit programm ord: 120
        elif(k[0] == 'x'):
            stopStep = 0
            for servoMotor in range(0,16):
                pwm.setPWM(servoMotor, 0, stopStep)
                time.sleep(0.1)
            step = 0
            print('[ LEGO-SERVO ] Steuerung wird beendet.!');
            exit();
# --------------------------------- EOF -------------------------------------------

