#!/bin/python 
#------------------------------------------------------------------------
# Robotik mit dem Raspberry Pi
# E.F.Engelhardt, Franzis Verlag, 2014
# -----------------------------------------------------------------------
# Das Skript nutzt die Motorenanschluesse des BrickPi A, B, C
# und wird ueber die Tasten
#  A = Basis links, D = Basis rechts, X = Ende
#  W = Neiger hoch, Y = Neiger runter           
#  Q = Greifer zu, E = Greifer auf               
#
# Datei: legokran-brickpi-step1.py
# Pfad: /Lego
# --------------------------------------------------------------------------------
from BrickPi import *
import os
import time
import threading
import struct
import sys
class _Getcharacter:
    def __init__(self):
        self.impl = _Getchar()
    def __call__(self):
        bytes = self.impl()
        if bytes == b'\x03':
            raise KeyboardInterrupt
        elif bytes == b'\x04':
            raise EOFError
        return bytes
# --------------------------------------------------------------------------------
class _Getchar:
    def __init__(self):
        import tty
        import sys
    def __call__(self):
        import sys
        import tty
        import termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch
# --------------------------------------------------------------------------------
getch = _Getcharacter()
# --------------------------------------------------------------------------------
def basis_turn_left():
    # neiger_hoch
    print('| Drehung nach links...                                        |');
    BrickPi.MotorSpeed[PORT_B] = -200
    update_and_wait()
    # dann drehung
    BrickPi.MotorSpeed[PORT_A] = 200
    update_and_wait()
# --------------------------------------------------------------------------------
def basis_turn_right():
    print('| Drehung nach rechts...                                       |');
    #neiger_hoch
    BrickPi.MotorSpeed[PORT_B] = -200
    update_and_wait()
    # dann drehung
    BrickPi.MotorSpeed[PORT_A] = -200
    update_and_wait()
# --------------------------------------------------------------------------------
def neiger_runter():
    print('| Neiger nach unten...                                          |');
    BrickPi.MotorSpeed[PORT_B] = 200
    update_and_wait()
# --------------------------------------------------------------------------------
def neiger_hoch():
    print('| Neiger nach oben                                             |');
    BrickPi.MotorSpeed[PORT_B] = -200
    update_and_wait()
# --------------------------------------------------------------------------------
def greifer_auf():
    print('| Greifer oeffnen                                              |');
    BrickPi.MotorSpeed[PORT_D] = 200 #0
    update_and_wait()
# --------------------------------------------------------------------------------
def greifer_zu():
    print('| Greifer schliessen                                           |');
    BrickPi.MotorSpeed[PORT_D] = -200 #0
    update_and_wait()
# --------------------------------------------------------------------------------
def update_and_wait():
    BrickPiUpdateValues();
    time.sleep(0.5)
# --------------------------------------------------------------------------------
def showControl():
    os.system('clear')
    print '|--------------------------------------------------------------|'
    print '| [ LEGO-KRAN ] A = Basis links, D = Basis rechts, X = Ende    |'
    print '|               W = Neiger hoch, Y = Neiger runter             |'
    print '|               Q = Greifer zu, E = Greifer auf                |'
    print '|--------------------------------------------------------------|'
# --------------------------------------------------------------------------------
# main
#
if __name__ == '__main__':
    BrickPiSetup() # Setup serial port communication
    BrickPi.MotorEnable[PORT_A] = 1 # Enable Motor A BASIS
    BrickPi.MotorEnable[PORT_B] = 1 # Enable Motor B NEIGER
    BrickPi.MotorEnable[PORT_D] = 1 # Enable Motor D GREIFER
    BrickPiSetupSensors() # Send Sensoreigenschaften zu BrickPi
    showControl(); #print 'A = Basis links, D = Basis rechts, X = Ende'
    inkey = _Getcharacter()
    while True:
        k=inkey()
        print(k[0])
        # 'a' turn left ord: 97
        if (k[0] == 'a'):
            basis_turn_left();
            showControl();
        # 'd' turn right ord: 100
        elif(k[0] == 'd'):
            basis_turn_right();
            showControl();
        # 'w' turn up ord: 119
        elif(k[0] == 'w'):
            neiger_hoch();
            showControl();
        # 'y' turn down ord: 121
        elif(k[0] == 'y'):
            neiger_runter();
            showControl();
        # 'q' turn close ord: 112
        elif(k[0] == 'q'):
            greifer_zu();
            showControl();
        # 'e' turn open ord: 101
        elif(k[0] == 'e'):
            greifer_auf();
            showControl();
        # 'x' exit programm ord: 120
        elif(k[0] == 'x'):
            print('[ LEGO-KRAN ] Steuerung wird beendet.!');
            exit();
# --------------------------------- EOF -------------------------------------------