# -*- coding: utf-8 -*- 
#!/usr/bin/env python
# -----------------------------------------------------------------------
# Robotik mit dem Raspberry Pi
# E.F.Engelhardt, Franzis Verlag, 2014
# -----------------------------------------------------------------------
# Pfad: /RaspiRoboCAR
# RaspiRoboCAR-step1.py
#-------------------------------------------------------------------------------
import RPi.GPIO as GPIO 
import time
import serial
import array
import os
import sys
import threading
import struct
#import tempfile
#import shutil
#import subprocess
#--------------------------------------------------------------------------------
class _Getcharacter:
    def __init__(self):
        self.impl = _Getchar()
    def __call__(self):
        bytes = self.impl()
        if bytes == b'\x03':
            raise KeyboardInterrupt
        elif bytes == b'\x04':
            raise EOFError
        return bytes
# --------------------------------------------------------------------------------
class _Getchar:
    def __init__(self):
        import tty
    def __call__(self):
        import tty
        import termios
        fd = sys.stdin.fileno()
        old_settings = termios.tcgetattr(fd)
        try:
            tty.setraw(sys.stdin.fileno())
            ch = sys.stdin.read(1)
        finally:
            termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return ch
# --------------------------------------------------------------------------------
getch = _Getcharacter()
# --------------------------------------------------------------------------------
def foto(destination=os.getcwd(),fileprefix="RaspiRoboCAR-CAM"):
       import tempfile
       import shutil
       import subprocess
       cam = subprocess.Popen(['/opt/vc/bin/vcgencmd', 'get_camera'],stdout=subprocess.PIPE)
       checkcam, err = cam.communicate()
       if int(checkcam[21]) == 1:
#          print "| Kamera angeschlossen! "
          photopath=tempfile.gettempdir() + '/' + fileprefix + '-' + time.strftime("%Y%m%d-%04d") + '.jpg'
#          print "-----------------------"
#          print "|      FOTO            |"
#          print "-----------------------"
          os.system ( "raspistill -n -t 500 -w 1024 -h 768 -e jpg -q 75 -o \"%s\"" % photopath )
          shutil.copy(photopath,destination)
       else:
          print "| Keine Kamera angeschlossen! "
       return
# --------------------------------------------------------------------------------
def showControl():
    os.system('clear')
    print '|----------------------------------------------------------------|'
    print '| [ RaspiRoboCAR ] a = Links,          d = Rechts                |'
    print '|                              s = Gerade                        |'
    print '|        q = Alle Motoren STOP,        x = Programmende          |'
    print '|                                                                |'
    print '|                W = Vorwaerts, Y = Rueckwaerts                  |'
    print '|----------------------------------------------------------------|'
# --------------------------------------------------------------------------------
def motor_on(en_pin, sec):
    GPIO.output(en_pin,GPIO.HIGH)
#        print "-----------------------"
#        print "| Antrieb an            |"
#        print "-----------------------"
    time.sleep(sec)
    return 
# --------------------------------------------------------------------------------
def motor_off(en_pin, sec):
    GPIO.output(en_pin,GPIO.LOW)
#        print "-----------------------"
#        print "| Antrieb aus           |"
#        print "-----------------------"
    time.sleep(sec)
    return 
# --------------------------------------------------------------------------------
def setup(in1a_antrieb_pin, in2a_antrieb_pin, ena_antrieb_pin, in3b_lenkung_pin, in4b_lenkung_pin, enb_lenkung_pin):
        #
        # Hier wird die PIN-Nummerierung des Raspberry Pi verwendet und NICHT die GPIO-Nummerierung
        GPIO.setmode(GPIO.BOARD)
        # Definition der Pins im Boardmode..
        GPIO.setup(ena_antrieb_pin, GPIO.OUT)
        GPIO.setup(in1a_antrieb_pin, GPIO.OUT)
        GPIO.setup(in2a_antrieb_pin, GPIO.OUT)
        GPIO.setup(enb_lenkung_pin, GPIO.OUT)
        GPIO.setup(in3b_lenkung_pin, GPIO.OUT)
        GPIO.setup(in4b_lenkung_pin, GPIO.OUT)
        print " ena_antrieb_pin auf ", ena_antrieb_pin, " gesetzt"
        print " in1a_antrieb_pin auf ", in1a_antrieb_pin, " gesetzt"
        print " in2a_antrieb_pin ", in2a_antrieb_pin, " gesetzt"
        print " enb_lenkung_pin ", enb_lenkung_pin, " gesetzt"
        print " in3b_lenkung_pin ", in3b_lenkung_pin, " gesetzt"
        print " in4b_lenkung_pin ", in4b_lenkung_pin, " gesetzt"
        dir = 6
        return
def motorrichtung(in1a_antrieb_pin, in2a_antrieb_pin, in3b_lenkung_pin, in4b_lenkung_pin, dir=6, units=1000): # vor/rueckwaerts
# 0 = vorwaerts / geradeaus
# 1 = rueckwaerts / geradeaus
# 2 = vorwaerts / links
# 3 = vorwaerts / rechts
# 4 = rueckwaerts / links
# 5 = rueckwaerts / rechts
        showControl();
        for step in range(0,units):
  #        motor_on(ena_antrieb_pin, 1);        
          print '| ---> DIR = ', dir , '                                                 |'
          if dir == 0:
                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Vorwaerts - gerade                                             |"
                GPIO.output(in1a_antrieb_pin,GPIO.LOW)
                print "| in1a_antrieb_pin auf LOW                                      |"
                GPIO.output(in2a_antrieb_pin,GPIO.HIGH)
                print "| in2a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in3b_lenkung_pin,GPIO.HIGH)
                GPIO.output(in4b_lenkung_pin,GPIO.HIGH)
                print "| in3b_lenkung_pin auf HIGH                                      |"
                print "| in4b_lenkung_pin auf HIGH                                      |"
          elif dir == 1:
                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Rueckwaerts - gerade                                           |" 
                GPIO.output(in1a_antrieb_pin,GPIO.HIGH)
                print "| in1a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in2a_antrieb_pin,GPIO.LOW)
                print "| in2a_antrieb_pin auf LOW                                      |"
                GPIO.output(in3b_lenkung_pin,GPIO.HIGH)
                GPIO.output(in4b_lenkung_pin,GPIO.HIGH)
                print "| in3b_lenkung_pin auf HIGH                                      |"
                print "| in4b_lenkung_pin auf HIGH                                      |"
          elif dir == 2:
#                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Vorwaerts - links                                              |"
                GPIO.output(in1a_antrieb_pin,GPIO.LOW)
                print "| in1a_antrieb_pin auf LOW                                      |"
                GPIO.output(in2a_antrieb_pin,GPIO.HIGH)
                print "| in2a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in3b_lenkung_pin,GPIO.LOW)
                GPIO.output(in4b_lenkung_pin,GPIO.HIGH)
                print "| in3b_lenkung_pin auf LOW                                       |"
                print "| in4b_lenkung_pin auf HIGH                                      |"
          elif dir == 3:
#                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Vorwaerts - rechts                                             |"
                GPIO.output(in1a_antrieb_pin,GPIO.LOW)
                print "| in1a_antrieb_pin auf LOW                                      |"
                GPIO.output(in2a_antrieb_pin,GPIO.HIGH)
                print "| in2a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in3b_lenkung_pin,GPIO.HIGH)
                GPIO.output(in4b_lenkung_pin,GPIO.LOW)
                print "| in3b_lenkung_pin auf HIGH                                      |"
                print "| in4b_lenkung_pin auf LOW                                       |"
          elif dir == 4:
                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Rueckwaerts - links"
                GPIO.output(in1a_antrieb_pin,GPIO.HIGH)
                print "| in1a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in2a_antrieb_pin,GPIO.LOW)
                print "| in2a_antrieb_pin auf LOW                                      |"
                GPIO.output(in3b_lenkung_pin,GPIO.HIGH)
                GPIO.output(in4b_lenkung_pin,GPIO.LOW)
                print "| in3b_lenkung_pin auf HIGH                                      |"
                print "| in4b_lenkung_pin auf LOW                                       |"
          elif dir == 5:
                motor_on(ena_antrieb_pin, .1);
                motor_on(enb_lenkung_pin, .1);
                print "| Rueckwaerts - rechts                                           |"
                GPIO.output(in1a_antrieb_pin,GPIO.HIGH)
                print "| in1a_antrieb_pin auf HIGH                                       |"
                GPIO.output(in2a_antrieb_pin,GPIO.LOW)
                print "| in2a_antrieb_pin auf LOW                                      |"
                GPIO.output(in3b_lenkung_pin,GPIO.LOW)
                GPIO.output(in4b_lenkung_pin,GPIO.HIGH)
                print "| in3b_lenkung_pin auf LOW                                       |"
                print "| in4b_lenkung_pin auf HIGH                                      |"
          else:
                motor_off(ena_antrieb_pin, .1);
                motor_off(enb_lenkung_pin, .1);
                print "| Alle Anschluesse auf HIGH -> STOP                              |"
                GPIO.output(in1a_antrieb_pin,GPIO.HIGH)
                print "| in1a_antrieb_pin auf HIGH                                      |"
                GPIO.output(in2a_antrieb_pin,GPIO.HIGH)
                print "| in2a_antrieb_pin auf HIGH                                      |"
                GPIO.output(in3b_lenkung_pin,GPIO.HIGH)
                GPIO.output(in4b_lenkung_pin,GPIO.HIGH)
                print "| in3b_lenkung_pin auf HIGH                                      |"
                print "| in4b_lenkung_pin auf HIGH                                      |"
                dir = 6
          print "|                                                                |"
          print "| Vor/Rueckwaerts-Richtung-Setzen erledigt                       |"
          print "-----------------------------------------------------------------|"
          return 
# -------------------------------------------------------------------------------------
# Definitionen 
# -------------------------------------------------------------------------------------
ena_antrieb_pin=22 
in1a_antrieb_pin=24 
in2a_antrieb_pin=21
enb_lenkung_pin=26
in3b_lenkung_pin=19
in4b_lenkung_pin=23
#
# ------------------------------------------------------------------------------------- 
# initialisieren
#
if __name__ == '__main__':
    setup(in1a_antrieb_pin, in2a_antrieb_pin, ena_antrieb_pin, in3b_lenkung_pin, in4b_lenkung_pin, enb_lenkung_pin)
    #  Dreh-Richtung einstellen
    #     richtung = 0
    showControl(); 
    inkey = _Getcharacter()
    while True:
        k=inkey()
        print(k[0])
        # 'a' ord: 97 # LINKS
        if (k[0] == 'a'):
            #
            #    LINKS
            # 
            dir = 2 # 2 links
        # 'd' ord: 100
        elif(k[0] == 'd'):
            # 
            #    RECHTS
            #
            dir = 3 # 3 rechts
        elif(k[0] == 's'):
            #
            #    GERADEAUS
            #
            dir = 0 # 0 geradeaus vorwaerts
        elif(k[0] == 'q'):
            #
            #    alles STOP
            #
            dir = 6; # 6 stop
            motor_off(ena_antrieb_pin, .1);
            motor_off(enb_lenkung_pin, .1);
        elif(k[0] == 'w'):
            #
            #    VORWAERTS 
            #
            dir = 0 # 0 geradeaus vorwaerts
        elif(k[0] == 'y'):
            #
            #    RUECKWAERTS 
            #
            dir = 1
        elif(k[0] == 'f'):
            #
            #    FOTO
            #
            foto(); # 0 geradeaus vorwaerts
        # 'x' exit programm ord: 120
#        elif(k[0] == '\x1b[A'):
        elif(k[0] == '^[[A'):
            print "up"
            dir = 0;
        elif(k[0] == '\x1b[B'):
            print "down"
            dir = 1;
        elif(k[0] == '\x1b[C'):
            print "right"
            dir = 3;
        elif(k[0] == '\x1b[D'):
            print "left"
            dir = 2;
        elif(k[0] == 'x'):
            print('[ RaspiRoboCAR ] Steuerung wird beendet.!');
            motor_off(ena_antrieb_pin, .1);
            motor_off(enb_lenkung_pin, .1);
            GPIO.cleanup();
            exit();
        motorrichtung(in1a_antrieb_pin, in2a_antrieb_pin, in3b_lenkung_pin, in4b_lenkung_pin, dir, 1000)
# ------------------------------ EOF ---------------------------------------------------
