# Published Jan 2013
# Revised Jan 2014 (to add new modules data)
# Author : Philippe Larduinat, philippelt@users.sourceforge.net
# Public domain source code
#
# Feb 2016
# Little Support for Welcome Cameras added
# Author : Erich Engelhardt, ef.engelhardt@gmx.de
#
# This API provides access to the Netatmo Welcome device
# This package can be used with Python2 or Python3 applications and do not
# require anything else than standard libraries

# PythonAPI Netatmo REST data access
# coding=utf-8

from sys import version_info
import json, time, datetime

# HTTP libraries depends upon Python 2 or 3
if version_info.major == 3 :
    import urllib.parse, urllib.request
else:
    from urllib import urlencode
    import urllib2

######################## USER SPECIFIC INFORMATION ######################

# To be able to have a program accessing your netatmo data, you have to register your program as
# a Netatmo app in your Netatmo account. All you have to do is to give it a name (whatever) and you will be
# returned a client_id and secret that your app has to supply to access netatmo servers.

_CLIENT_ID     = "XXXXXXXXXXXXXXXXXXXXX"
_CLIENT_SECRET = "XXXXXXXXXXXXXXXXXXXXX"
_USERNAME      = "XXXXXXXXXXXXXXX@XXXXXXXXXXXXXXX.com"
_PASSWORD      = "XXXXXXXXXXXXXXXXXXXXX"

#########################################################################


# Common definitions
_BASE_URL       = "https://api.netatmo.net/"
_AUTH_REQ       = _BASE_URL + "oauth2/token"

# WELCOME METHODS
_GETHOMEDATA_REQ = _BASE_URL + "api/gethomedata"
_GETNEXTEVENTS_REQ = _BASE_URL + "api/getnextevents"
_GETLASTEVENTOF_REQ = _BASE_URL + "api/getlasteventof"
_GETEVENTSUNTIL_REQ = _BASE_URL + "api/geteventsuntil"
_GETCAMERAPICTURE_REQ = _BASE_URL + "api/getcamerapicture"

class ClientAuth:
    "Request authentication and keep access token available through token method. Renew it automatically if necessary"

    def __init__(self, clientId=_CLIENT_ID,
                       clientSecret=_CLIENT_SECRET,
                       username=_USERNAME,
                       password=_PASSWORD):

        postParams = {
                "grant_type" : "password",
                "client_id" : clientId,
                "client_secret" : clientSecret,
                "username" : username,
                "password" : password,
                "scope" : "read_camera" # scope for welcome
                }

        resp = postRequest(_AUTH_REQ, postParams)

        self._clientId = clientId
        self._clientSecret = clientSecret
        self._accessToken = resp['access_token']
        self.refreshToken = resp['refresh_token']
        self._scope = resp['scope']
        self.expiration = int(resp['expire_in'] + time.time())

    @property
    def accessToken(self):
        if self.expiration < time.time(): # Token should be renewed
            postParams = {
                    "grant_type" : "refresh_token",
                    "refresh_token" : self.refreshToken,
                    "client_id" : self._clientId,
                    "client_secret" : self._clientSecret
                    }
            resp = postRequest(_AUTH_REQ, postParams)

            self._accessToken = resp['access_token']
            self.refreshToken = resp['refresh_token']
            self.expiration = int(resp['expire_in'] + time.time())
        return self._accessToken

class User:
    def __init__(self, authData):
        postParams = {
                "access_token" : authData.accessToken
                }
        resp = postRequest(_GETUSER_REQ, postParams)
        self.rawData = resp['body']
        self.id = self.rawData['_id']
        self.name = self.rawData['name']

class Welcome:
    def __init__(self, authData):
        self.getAuthToken = authData.accessToken
        postParams = {
                "access_token" : authData.accessToken
                }
        resp = postRequest(_GETHOMEDATA_REQ, postParams)
        self.rawData = resp['body']
        self.homes = { h['name'] : h for h in self.rawData['homes'] }
        print "... Welcome-Init... done..."
        print "---------------------------"

    def getlasteventofPseudo(self,pseudo):
        self.person_id = self.IDpseudo(pseudo)        
        self.home_id = self.getHomeid()
        self.lasteventof = self.getlasteventof(self.home_id,self.person_id)
        return self.lasteventof

    def getHomeid(self):
        self.homeid = list(self.homes.values())[0]['id']       
        return self.homeid

    def showEventsListID(self):
        self.events = list(self.homes.values())[0]['events']
        j = 0
        for i in self.events:
#            print "Ereignis: %s \tid=%s\tPerson-ID: %s" %(j, list(self.events)[j]['id'], list(self.events)[j]['person_id'])
            print "Ereignis: %s \tid=%s" %(j, list(self.events)[j]['id'])
            j +=1
        return j

    def getLastSeenpseudo(self,pseudo=None):
        # print "Suche von %s" %pseudo
        self.persons = list(self.homes.values())[0]['persons']
        j = 0
        found = False
        for i in self.persons:
           try:
              if list(self.persons)[j]['pseudo'] == pseudo:
                 self.personlastseen = datetime.datetime.fromtimestamp(int(list(self.persons)[j]['last_seen'])).strftime('%d-%m-%Y %H:%M:%S')
                 found = True
           except KeyError:
              self.personlastseen = 0
              pass
           j += 1
           if found:
              break
        return self.personlastseen

    def IDpseudo(self,pseudo=None):
        # print "Suche von %s" %pseudo
        self.persons = list(self.homes.values())[0]['persons']
        j = 0
        found = False
        for i in self.persons:
           try:
              if list(self.persons)[j]['pseudo'] == pseudo:
                 self.personid = list(self.persons)[j]['id']
                 found = True
           except KeyError:
              self.personid = 0
              pass
           j += 1
           if found:
              break
        return self.personid

    def showPersonsListLastseenPseudo(self):
        self.persons = list(self.homes.values())[0]['persons']        
        j = 0
        for i in self.persons:
           print "\nid: %s\tlast seen: %s" %(list(self.persons)[j]['id'], datetime.datetime.fromtimestamp(int(list(self.persons)[j]['last_seen'])).strftime('%d-%m-%Y %H:%M:%S'))
           try:
              print " Pseudo-Name: %s" %(list(self.persons)[j]['pseudo']) # in self.persons:
           except KeyError:
              pass
           j +=1
        return 1

    def showPersonsListID(self):
        self.persons = list(self.homes.values())[0]['persons']
        j = 0
        for i in self.persons:
            print "Person\t %s \tid=%s " %(j, list(self.persons)[j]['id'])
            j +=1
        return j

    def getHomename(self):
        self.homename = list(self.homes.values())[0]['name']
        return self.homename

    def getCamname(self):
        self.camname = list(list(self.homes.values())[0]['cameras'])[0]['name']
        return self.camname

    def getHomedata(self):
        postParams = { "access_token" : self.getAuthToken }
        return postRequest(_GETHOMEDATA_REQ, postParams)

    def getNextevents(self,home_id,event_id, size=None):
        postParams = { "access_token" : self.getAuthToken }
        postParams['home_id']  = home_id
        postParams['event_id']  = event_id
        if size : postParams['size'] = size
        return postRequest(_GETNEXTEVENTS_REQ, postParams)

    def getlasteventof(self,home_id,person_id, offset=None):               
        print"home-ID= %s\n" %home_id
        print"person-ID= %s\n" %person_id
        postParams = { "access_token" : self.getAuthToken }
        postParams['home_id']  = home_id
        postParams['person_id']  = person_id
        if offset : postParams['offset'] = offset
        return postRequest(_GETLASTEVENTOF_REQ, postParams)

    def geteventsuntil(self,home_id,event_id):
        postParams = { "access_token" : self.getAuthToken }
        postParams['home_id']  = home_id
        postParams['event_id']  = event_id
        return postRequest(_GETEVENTSUNTIL_REQ, postParams)

    def getcamerapicture(self,image_id,key):
        postParams = { "access_token" : self.getAuthToken }
        postParams['image_id']  = image_id
        postParams['key']  = key
        return postRequest(_GETCAMERAPICTURE_REQ, postParams)


# Utilities routines
def postRequest(url, params):
    if version_info.major == 3:
        req = urllib.request.Request(url)
        req.add_header("Content-Type","application/x-www-form-urlencoded;charset=utf-8")
        params = urllib.parse.urlencode(params).encode('utf-8')
        resp = urllib.request.urlopen(req, params).readall().decode("utf-8")
    else:
        params = urlencode(params)
        headers = {"Content-Type" : "application/x-www-form-urlencoded;charset=utf-8"}
        req = urllib2.Request(url=url, data=params, headers=headers)
        resp = urllib2.urlopen(req).read()
    return json.loads(resp)

def toTimeString(value):
    return time.strftime("%Y-%m-%d_%H:%M:%S", time.localtime(int(value)))

def toEpoch(value):
    return int(time.mktime(time.strptime(value,"%Y-%m-%d_%H:%M:%S")))

def todayStamps():
    today = time.strftime("%Y-%m-%d")
    today = int(time.mktime(time.strptime(today,"%Y-%m-%d")))
    return today, today+3600*24

def getIDpseudo(pseudo=None):
    return Welcome.IDpseudo


# auto-test when executed directly
if __name__ == "__main__":

    from sys import exit, stdout, stderr
    
    if not _CLIENT_ID or not _CLIENT_SECRET or not _USERNAME or not _PASSWORD :
           stderr.write("Library source missing identification arguments to check lwelcome.py (user/password/etc...)")
           exit(1)
    
    authorization = ClientAuth()                # Test authentication method
    # If we reach this line, all is OK
    
    # If launched interactively, display OK message
    if stdout.isatty():
        print("lwelcome.py : OK")
    
    exit(0)
